@@ -2,7 +2,7 @@
  *
  *  grtypes.h
  *
- *    basic type defintions
+ *    basic type definitions
  *
  *  Copyright 1999 - The FreeType Development Team - www.freetype.org
  *
@@ -11,8 +11,8 @@
  *
  ***************************************************************************/
 
-#ifndef GRTYPES_H
-#define GRTYPES_H
+#ifndef GRTYPES_H_
+#define GRTYPES_H_
 
   typedef unsigned char  byte;
 
@@ -49,4 +49,4 @@
 #define  GR_LOCAL_FUNC    /* void */
 #endif
 
-#endif /* GRTYPES_H */
+#endif /* GRTYPES_H_ */
