@@ -68,6 +68,9 @@
   /* default gamma setting */
 #define GAMMA  1.8
 
+  /* special encoding to display glyphs in order */
+#define FT_ENCODING_ORDER  0xFFFF
+
 #include "graph.h"
 #include "grobjs.h"
 #include "grfont.h"
@@ -79,6 +82,7 @@
     grColor     fore_color;
     grColor     back_color;
     grColor     warn_color;
+    double      gamma;
 
   } FTDemo_Display;
 
@@ -134,8 +138,10 @@
   } TFont, *PFont;
 
   enum {
-    LCD_MODE_AA = 0,
+    LCD_MODE_MONO = 0,
+    LCD_MODE_AA,
     LCD_MODE_LIGHT,
+    LCD_MODE_LIGHT_SUBPIXEL,
     LCD_MODE_RGB,
     LCD_MODE_BGR,
     LCD_MODE_VRGB,
@@ -189,12 +195,10 @@
 
     /* call FTDemo_Update_Current_Flags after setting any of the following fields */
     int             hinted;            /* is glyph hinting active?    */
-    int             antialias;         /* is anti-aliasing active?    */
     int             use_sbits;         /* do we use embedded bitmaps? */
     int             autohint;          /* force auto-hinting          */
-    int             lcd_mode;
+    int             lcd_mode;          /* mono, aa, light, vrgb, ...  */
     int             preload;           /* force font file preloading  */
-    int             color;             /* load color bitmaps          */
 
     /* don't touch the following fields! */
 
@@ -203,7 +207,7 @@
     int             string_length;
     int             string_reload;
 
-    FT_Encoding     encoding;
+    unsigned long   encoding;
     FT_Stroker      stroker;
     FT_Bitmap       bitmap;            /* used as bitmap conversion buffer */
 
@@ -222,7 +226,8 @@
   FT_Error
   FTDemo_Install_Font( FTDemo_Handle*  handle,
                        const char*     filepath,
-                       FT_Bool         outline_only );
+                       FT_Bool         outline_only,
+                       FT_Bool         no_instances );
 
 
   void
@@ -258,6 +263,16 @@
                    FT_Size*        asize );
 
 
+  /* draw common header */
+  void
+  FTDemo_Draw_Header( FTDemo_Handle*   handle,
+                      FTDemo_Display*  display,
+                      int              ptsize,
+                      int              res,
+                      int              idx,
+                      int              error_code );
+
+
   /* convert a FT_Glyph to a grBitmap (don't free target->buffer) */
   /* if aglyf != NULL, you should FT_Glyph_Done the aglyf */
   FT_Error
@@ -334,7 +349,7 @@
 
 
   /* make a FT_Encoding tag from a string */
-  FT_Encoding
+  unsigned long
   FTDemo_Make_Encoding_Tag( const char*  s );
 
 
