@@ -2,7 +2,7 @@
 /*                                                                          */
 /*  The FreeType project -- a free and portable quality TrueType renderer.  */
 /*                                                                          */
-/*  Copyright 1996-2000, 2003-2007, 2010, 2012-2013                         */
+/*  Copyright 1996-2017                                                     */
 /*  D. Turner, R.Wilhelm, and W. Lemberg                                    */
 /*                                                                          */
 /****************************************************************************/
@@ -13,6 +13,7 @@
 #include FT_SFNT_NAMES_H
 #include FT_TRUETYPE_IDS_H
 #include FT_TRUETYPE_TABLES_H
+#include FT_TRUETYPE_TAGS_H
 #include FT_MULTIPLE_MASTERS_H
 
   /* the following header shouldn't be used in normal programs */
@@ -23,8 +24,15 @@
 #include FT_INTERNAL_OBJECTS_H
 #include FT_INTERNAL_DRIVER_H
 
+  /* error messages */
+#undef FTERRORS_H_
+#define FT_ERROR_START_LIST     {
+#define FT_ERRORDEF( e, v, s )  case v: str = s; break;
+#define FT_ERROR_END_LIST       default: str = "unknown error"; }
+
 #include "common.h"
 #include "output.h"
+#include "mlgetopt.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -36,14 +44,25 @@
   static int  comma_flag  = 0;
   static int  verbose     = 0;
   static int  name_tables = 0;
+  static int  bytecode    = 0;
+  static int  tables      = 0;
   static int  utf8        = 0;
 
 
   /* PanicZ */
   static void
-  PanicZ( const char*  message )
+  PanicZ( FT_Library   library,
+          const char*  message )
   {
-    fprintf( stderr, "%s\n  error = 0x%04x\n", message, error );
+    const FT_String  *str;
+
+
+    FT_Done_FreeType( library );
+
+    switch( error )
+    #include FT_ERRORS_H
+
+    fprintf( stderr, "%s\n  error = 0x%04x, %s\n", message, error, str );
     exit( 1 );
   }
 
@@ -60,8 +79,11 @@
 
 
   static void
-  usage( char*  execname )
+  usage( FT_Library  library,
+         char*       execname )
   {
+    FT_Done_FreeType( library );
+
     fprintf( stderr,
       "\n"
       "ftdump: simple font dumper -- part of the FreeType project\n"
@@ -73,6 +95,8 @@
 
     fprintf( stderr,
       "  -n        Print SFNT name tables.\n"
+      "  -p        Print TrueType programs.\n"
+      "  -t        Print SFNT table list.\n"
       "  -u        Emit UTF8.\n"
       "  -V        Be verbose.\n"
       "\n"
@@ -185,64 +209,57 @@
   }
 
 
+#define XEXPAND( x )  #x
+#define EXPAND( x )   XEXPAND( x )
+
+#define NAME_ID( tag, description ) \
+          case TT_NAME_ID_ ## tag: \
+            return description " (ID " EXPAND( TT_NAME_ID_ ## tag ) ")"
+
+
   static const char*
   name_id( int  id )
   {
     switch ( id )
     {
-    case TT_NAME_ID_COPYRIGHT:
-      return "copyright";
-    case TT_NAME_ID_FONT_FAMILY:
-      return "font family";
-    case TT_NAME_ID_FONT_SUBFAMILY:
-      return "font subfamily";
-    case TT_NAME_ID_UNIQUE_ID:
-      return "unique ID";
-    case TT_NAME_ID_FULL_NAME:
-      return "full name";
-    case TT_NAME_ID_VERSION_STRING:
-      return "version string";
-    case TT_NAME_ID_PS_NAME:
-      return "PostScript name";
-    case TT_NAME_ID_TRADEMARK:
-      return "trademark";
+      NAME_ID( COPYRIGHT, "copyright" );
+      NAME_ID( FONT_FAMILY, "font family" );
+      NAME_ID( FONT_SUBFAMILY, "font subfamily" );
+      NAME_ID( UNIQUE_ID, "unique font identifier" );
+      NAME_ID( FULL_NAME, "full name" );
+      NAME_ID( VERSION_STRING, "version string" );
+      NAME_ID( PS_NAME, "PostScript name" );
+      NAME_ID( TRADEMARK, "trademark" );
 
-    /* the following values are from the OpenType spec */
-    case TT_NAME_ID_MANUFACTURER:
-      return "manufacturer";
-    case TT_NAME_ID_DESIGNER:
-      return "designer";
-    case TT_NAME_ID_DESCRIPTION:
-      return "description";
-    case TT_NAME_ID_VENDOR_URL:
-      return "vendor URL";
-    case TT_NAME_ID_DESIGNER_URL:
-      return "designer URL";
-    case TT_NAME_ID_LICENSE:
-      return "license";
-    case TT_NAME_ID_LICENSE_URL:
-      return "license URL";
-    /* number 15 is reserved */
-    case TT_NAME_ID_PREFERRED_FAMILY:
-      return "preferred family";
-    case TT_NAME_ID_PREFERRED_SUBFAMILY:
-      return "preferred subfamily";
-    case TT_NAME_ID_MAC_FULL_NAME:
-      return "Mac full name";
+      /* the following values are from the OpenType spec */
+      NAME_ID( MANUFACTURER, "manufacturer" );
+      NAME_ID( DESIGNER, "designer" );
+      NAME_ID( DESCRIPTION, "description" );
+      NAME_ID( VENDOR_URL, "vendor URL" );
+      NAME_ID( DESIGNER_URL, "designer URL" );
+      NAME_ID( LICENSE, "license" );
+      NAME_ID( LICENSE_URL, "license URL" );
+      /* number 15 is reserved */
+      NAME_ID( TYPOGRAPHIC_FAMILY, "typographic family" );
+      NAME_ID( TYPOGRAPHIC_SUBFAMILY, "typographic subfamily" );
+      NAME_ID( MAC_FULL_NAME, "Mac full name" );
 
-    /* the following code is new as of 2000-01-21 */
-    case TT_NAME_ID_SAMPLE_TEXT:
-      return "sample text";
+      /* the following code is new as of 2000-01-21 */
+      NAME_ID( SAMPLE_TEXT, "sample text" );
 
-    /* this is new in OpenType 1.3 */
-    case TT_NAME_ID_CID_FINDFONT_NAME:
-      return "CID `findfont' name";
+      /* this is new in OpenType 1.3 */
+      NAME_ID( CID_FINDFONT_NAME, "CID `findfont' name" );
 
-    /* this is new in OpenType 1.5 */
-    case TT_NAME_ID_WWS_FAMILY:
-      return "WWS family name";
-    case TT_NAME_ID_WWS_SUBFAMILY:
-      return "WWS subfamily name";
+      /* this is new in OpenType 1.5 */
+      NAME_ID( WWS_FAMILY, "WWS family name" );
+      NAME_ID( WWS_SUBFAMILY, "WWS subfamily name" );
+
+      /* this is new in OpenType 1.7 */
+      NAME_ID( LIGHT_BACKGROUND, "light background palette" );
+      NAME_ID( DARK_BACKGROUND, "dark background palette" );
+
+      /* this is new in OpenType 1.8 */
+      NAME_ID( VARIATIONS_PREFIX, "variations PostScript name prefix" );
 
     default:
       return NULL;
@@ -340,8 +357,7 @@
 
           switch ( name.encoding_id )
           {
-            /* TT_MS_ID_SYMBOL_CS is supposed to be Unicode, according to */
-            /* information from the MS font development team              */
+            /* TT_MS_ID_SYMBOL_CS is Unicode, similar to PID/EID=3/1 */
           case TT_MS_ID_SYMBOL_CS:
           case TT_MS_ID_UNICODE_CS:
             put_unicode_be16( name.string, name.string_len, 6, utf8 );
@@ -365,6 +381,43 @@
   }
 
 
+  static void
+  Print_Sfnt_Tables( FT_Face  face )
+  {
+    FT_ULong  num_tables, i;
+    FT_ULong  tag, length;
+    FT_Byte   buffer[4];
+
+
+    FT_Sfnt_Table_Info( face, 0, NULL, &num_tables );
+
+    printf( "font tables (%lu)\n", num_tables );
+
+    for ( i = 0; i < num_tables; i++ )
+    {
+      FT_Sfnt_Table_Info( face, (FT_UInt)i, &tag, &length );
+
+      if ( length >= 4 )
+      {
+        length = 4;
+        FT_Load_Sfnt_Table( face, tag, 0, buffer, &length );
+      }
+      else
+        continue;
+
+      printf( "  %2lu: %c%c%c%c %02X%02X%02X%02X...\n", i,
+                                   (FT_Char)( tag >> 24 ),
+                                   (FT_Char)( tag >> 16 ),
+                                   (FT_Char)( tag >>  8 ),
+                                   (FT_Char)( tag ),
+                                       (FT_UInt)buffer[0],
+                                       (FT_UInt)buffer[1],
+                                       (FT_UInt)buffer[2],
+                                       (FT_UInt)buffer[3] );
+    }
+  }
+
+
   static void
   Print_Fixed( FT_Face  face )
   {
@@ -399,17 +452,25 @@
       active = FT_Get_Charmap_Index( face->charmap );
 
     /* CharMaps */
-    printf( "charmaps\n" );
+    printf( "charmaps (%d)\n", face->num_charmaps );
 
     for( i = 0; i < face->num_charmaps; i++ )
     {
+      FT_Long   format  = FT_Get_CMap_Format( face->charmaps[i] );
       FT_ULong  lang_id = FT_Get_CMap_Language_ID( face->charmaps[i] );
 
 
-      printf( "   %d: platform %u, encoding %2u",
-              i,
-              face->charmaps[i]->platform_id,
-              face->charmaps[i]->encoding_id );
+      if ( format >= 0 )
+        printf( "  %2d: format %2ld, platform %u, encoding %2u",
+                i,
+                format,
+                face->charmaps[i]->platform_id,
+                face->charmaps[i]->encoding_id );
+      else
+        printf( "  %2d: synthetic, platform %u, encoding %2u",
+                i,
+                face->charmaps[i]->platform_id,
+                face->charmaps[i]->encoding_id );
 
       if ( lang_id == 0xFFFFFFFFUL )
         printf( "   (Unicode Variation Sequences)" );
@@ -424,8 +485,9 @@
 
       if ( verbose )
       {
-        FT_ULong  charcode;
-        FT_UInt   gindex;
+        FT_ULong   charcode;
+        FT_UInt    gindex;
+        FT_String  buf[32];
 
 
         FT_Set_Charmap( face, face->charmaps[i] );
@@ -433,7 +495,12 @@
         charcode = FT_Get_First_Char( face, &gindex );
         while ( gindex )
         {
-          printf( "      0x%04lx => %d\n", charcode, gindex );
+          if ( FT_HAS_GLYPH_NAMES ( face ) )
+            FT_Get_Glyph_Name( face, gindex, buf, 32 );
+          else
+            buf[0] = '\0';
+
+          printf( "      0x%04lx => %d %s\n", charcode, gindex, buf );
           charcode = FT_Get_Next_Char( face, charcode, &gindex );
         }
         printf( "\n" );
@@ -521,6 +588,188 @@
   }
 
 
+  static void
+  Print_Bytecode( FT_Byte*   buffer,
+                  FT_UShort  length,
+                  char*      tag )
+  {
+    FT_UShort  i;
+    int        j = 0;  /* status counter */
+
+
+    for ( i = 0; i < length; i++ )
+    {
+      if ( ( i & 15 ) == 0 )
+        printf( "\n%s:%04hx ", tag, i );
+
+      if ( j == 0 )
+      {
+        printf( " %02x", (FT_UInt)buffer[i] );
+
+        if ( buffer[i] == 0x40 )
+          j = -1;
+        else if ( buffer[i] == 0x41 )
+          j = -2;
+        else if ( 0xB0 <= buffer[i] && buffer[i] <= 0xB7 )
+          j = buffer[i] - 0xAF;
+        else if ( 0xB8 <= buffer[i] && buffer[i] <= 0xBF )
+          j = 2 * ( buffer[i] - 0xB7 );
+      }
+      else
+      {
+        printf( "_%02x", (FT_UInt)buffer[i] );
+
+        if ( j == -1 )
+          j = buffer[i];
+        else if ( j == -2 )
+          j = 2 * buffer[i];
+        else
+          j--;
+      }
+    }
+    printf( "\n" );
+  }
+
+
+  static void
+  Print_Programs( FT_Face face )
+  {
+    FT_ULong    length = 0;
+    FT_UShort   i;
+    FT_Byte*    buffer = NULL;
+    FT_Byte*    offset = NULL;
+
+    TT_Header*      head;
+    TT_MaxProfile*  maxp;
+
+
+    error = FT_Load_Sfnt_Table( face, TTAG_fpgm, 0, NULL, &length );
+    if ( error || length == 0 )
+      goto Prep;
+
+    buffer = (FT_Byte*)malloc( length );
+    if ( buffer == NULL )
+      goto Exit;
+
+    error = FT_Load_Sfnt_Table( face, TTAG_fpgm, 0, buffer, &length );
+    if ( error )
+      goto Exit;
+
+    printf( "font program" );
+    Print_Bytecode( buffer, (FT_UShort)length, (char*)"fpgm" );
+
+  Prep:
+    length = 0;
+
+    error = FT_Load_Sfnt_Table( face, TTAG_prep, 0, NULL, &length );
+    if ( error || length == 0 )
+      goto Glyf;
+
+    buffer = (FT_Byte*)realloc( buffer, length );
+    if ( buffer == NULL )
+      goto Exit;
+
+    error = FT_Load_Sfnt_Table( face, TTAG_prep, 0, buffer, &length );
+    if ( error )
+      goto Exit;
+
+    printf( "\ncontrol value program" );
+    Print_Bytecode( buffer, (FT_UShort)length, (char*)"prep" );
+
+  Glyf:
+    length = 0;
+
+    error = FT_Load_Sfnt_Table( face, TTAG_glyf, 0, NULL, &length );
+    if ( error || length == 0 )
+      goto Exit;
+
+    buffer = (FT_Byte*)realloc( buffer, length );
+    if ( buffer == NULL )
+      goto Exit;
+
+    error = FT_Load_Sfnt_Table( face, TTAG_glyf, 0, buffer, &length );
+    if ( error )
+      goto Exit;
+
+    length = 0;
+
+    error = FT_Load_Sfnt_Table( face, TTAG_loca, 0, NULL, &length );
+    if ( error || length == 0 )
+      goto Exit;
+
+    offset = (FT_Byte*)malloc( length );
+    if ( offset == NULL )
+      goto Exit;
+
+    error = FT_Load_Sfnt_Table( face, TTAG_loca, 0, offset, &length );
+    if ( error )
+      goto Exit;
+
+    head =     (TT_Header*)FT_Get_Sfnt_Table( face, FT_SFNT_HEAD );
+    maxp = (TT_MaxProfile*)FT_Get_Sfnt_Table( face, FT_SFNT_MAXP );
+
+    for ( i = 0; i < maxp->numGlyphs; i++ )
+    {
+      FT_UInt32  loc;
+      FT_UInt16  len;
+      char       tag[5];
+
+
+      if ( head->Index_To_Loc_Format )
+        loc = (FT_UInt32)offset[4 * i    ] << 24 |
+              (FT_UInt32)offset[4 * i + 1] << 16 |
+              (FT_UInt32)offset[4 * i + 2] << 8  |
+              (FT_UInt32)offset[4 * i + 3];
+      else
+        loc = (FT_UInt32)offset[2 * i    ] << 9 |
+              (FT_UInt32)offset[2 * i + 1] << 1;
+
+      len = (FT_UInt16)( buffer[loc] << 8 | buffer[loc + 1] );
+
+      loc += 10;
+
+      if ( (FT_Int16)len < 0 )  /* composite */
+      {
+        FT_UShort  flags;
+
+
+        do
+        {
+          flags = (FT_UInt16)( buffer[loc] << 8 | buffer[loc + 1] );
+
+          loc += 4;
+
+          loc += flags & FT_SUBGLYPH_FLAG_ARGS_ARE_WORDS ? 4 : 2;
+
+          loc += flags & FT_SUBGLYPH_FLAG_SCALE ? 2
+                   : flags & FT_SUBGLYPH_FLAG_XY_SCALE ? 4
+                       : flags & FT_SUBGLYPH_FLAG_2X2 ? 8 : 0;
+        } while ( flags & 0x20 );  /* more components */
+
+        if ( ( flags & 0x100 ) == 0 )
+          continue;
+      }
+      else
+        loc += 2 * len;
+
+      len = (FT_UInt16)( buffer[loc] << 8 | buffer[loc + 1] );
+
+      if ( len == 0 )
+        continue;
+
+      loc += 2;
+
+      sprintf( tag, "%04hx", i );
+      printf("\nglyf program %hd (%.4s)", i, tag );
+      Print_Bytecode( buffer + loc, len, tag );
+    }
+
+  Exit:
+    free( buffer );
+    free( offset );
+  }
+
+
   int
   main( int    argc,
         char*  argv[] )
@@ -541,11 +790,11 @@
     /* Initialize engine */
     error = FT_Init_FreeType( &library );
     if ( error )
-      PanicZ( "Could not initialize FreeType library" );
+      PanicZ( library, "Could not initialize FreeType library" );
 
     while ( 1 )
     {
-      option = getopt( argc, argv, "nuvV" );
+      option = getopt( argc, argv, "nptuvV" );
 
       if ( option == -1 )
         break;
@@ -556,6 +805,14 @@
         name_tables = 1;
         break;
 
+      case 'p':
+        bytecode = 1;
+        break;
+
+      case 't':
+        tables = 1;
+        break;
+
       case 'u':
         utf8 = 1;
         break;
@@ -580,7 +837,7 @@
         break;
 
       default:
-        usage( execname );
+        usage( library, execname );
         break;
       }
     }
@@ -589,7 +846,7 @@
     argv += optind;
 
     if ( argc != 1 )
-      usage( execname );
+      usage( library, execname );
 
     file = 0;
 
@@ -623,7 +880,7 @@
     /* Load face */
     error = FT_New_Face( library, filename, 0, &face );
     if ( error )
-      PanicZ( "Could not open face." );
+      PanicZ( library, "Could not open face." );
 
   Success:
     num_faces = face->num_faces;
@@ -638,7 +895,7 @@
     {
       error = FT_New_Face( library, filename, i, &face );
       if ( error )
-        PanicZ( "Could not open face." );
+        PanicZ( library, "Could not open face." );
 
       printf( "\n----- Face number: %d -----\n\n", i );
       Print_Name( face );
@@ -653,6 +910,18 @@
         Print_Sfnt_Names( face );
       }
 
+      if ( tables && FT_IS_SFNT( face ) )
+      {
+        printf( "\n" );
+        Print_Sfnt_Tables( face );
+      }
+
+      if ( bytecode && FT_IS_SFNT( face ) )
+      {
+        printf( "\n" );
+        Print_Programs( face );
+      }
+
       if ( face->num_fixed_sizes )
       {
         printf( "\n" );
