@@ -2,7 +2,7 @@
 /*                                                                          */
 /*  The FreeType project -- a free and portable quality TrueType renderer.  */
 /*                                                                          */
-/*  Copyright 1996-2007, 2009-2015 by                                       */
+/*  Copyright 1996-2007, 2009-2016 by                                       */
 /*  D. Turner, R.Wilhelm, and W. Lemberg                                    */
 /*                                                                          */
 /*                                                                          */
@@ -13,6 +13,7 @@
 
 #include "ftcommon.h"
 #include "common.h"
+#include "mlgetopt.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -26,7 +27,49 @@
 #define MAXPTSIZE          500   /* dtp */
 
 
-  static char*  Text = (char *)"The quick brown fox jumps over the lazy dog";
+  static const char*  Sample[] =
+  {
+    "The quick brown fox jumps over the lazy dog",
+
+    /* Luís argüia à Júlia que «brações, fé, chá, óxido, pôr, zângão» */
+    /* eram palavras do português */
+    "Lu\u00EDs arg\u00FCia \u00E0 J\u00FAlia que \u00ABbra\u00E7\u00F5es, "
+    "f\u00E9, ch\u00E1, \u00F3xido, p\u00F4r, z\u00E2ng\u00E3o\u00BB eram "
+    "palavras do portugu\u00EAs",
+
+    /* Ο καλύμνιος σφουγγαράς ψιθύρισε πως θα βουτήξει χωρίς να διστάζει */
+    "\u039F \u03BA\u03B1\u03BB\u03CD\u03BC\u03BD\u03B9\u03BF\u03C2 \u03C3"
+    "\u03C6\u03BF\u03C5\u03B3\u03B3\u03B1\u03C1\u03AC\u03C2 \u03C8\u03B9"
+    "\u03B8\u03CD\u03C1\u03B9\u03C3\u03B5 \u03C0\u03C9\u03C2 \u03B8\u03B1 "
+    "\u03B2\u03BF\u03C5\u03C4\u03AE\u03BE\u03B5\u03B9 \u03C7\u03C9\u03C1"
+    "\u03AF\u03C2 \u03BD\u03B1 \u03B4\u03B9\u03C3\u03C4\u03AC\u03B6\u03B5"
+    "\u03B9",
+
+    /* Съешь ещё этих мягких французских булок да выпей же чаю */
+    "\u0421\u044A\u0435\u0448\u044C \u0435\u0449\u0451 \u044D\u0442\u0438"
+    "\u0445 \u043C\u044F\u0433\u043A\u0438\u0445 \u0444\u0440\u0430\u043D"
+    "\u0446\u0443\u0437\u0441\u043A\u0438\u0445 \u0431\u0443\u043B\u043E"
+    "\u043A \u0434\u0430 \u0432\u044B\u043F\u0435\u0439 \u0436\u0435 "
+    "\u0447\u0430\u044E",
+
+    /* 天地玄黃，宇宙洪荒。日月盈昃，辰宿列張。寒來暑往，秋收冬藏。*/
+    "\u5929\u5730\u7384\u9EC3\uFF0C\u5B87\u5B99\u6D2A\u8352\u3002\u65E5"
+    "\u6708\u76C8\u6603\uFF0C\u8FB0\u5BBF\u5217\u5F35\u3002\u5BD2\u4F86"
+    "\u6691\u5F80\uFF0C\u79CB\u6536\u51AC\u85CF\u3002",
+
+    /* いろはにほへと ちりぬるを わかよたれそ つねならむ */
+    /* うゐのおくやま けふこえて あさきゆめみし ゑひもせす */
+    "\u3044\u308D\u306F\u306B\u307B\u3078\u3068 \u3061\u308A\u306C\u308B"
+    "\u3092 \u308F\u304B\u3088\u305F\u308C\u305D \u3064\u306D\u306A\u3089"
+    "\u3080 \u3046\u3090\u306E\u304A\u304F\u3084\u307E \u3051\u3075\u3053"
+    "\u3048\u3066 \u3042\u3055\u304D\u3086\u3081\u307F\u3057 \u3091\u3072"
+    "\u3082\u305B\u3059",
+
+    /* 키스의 고유조건은 입술끼리 만나야 하고 특별한 기술은 필요치 않다 */
+    "\uD0A4\uC2A4\uC758 \uACE0\uC720\uC870\uAC74\uC740 \uC785\uC220\uB07C"
+    "\uB9AC \uB9CC\uB098\uC57C \uD558\uACE0 \uD2B9\uBCC4\uD55C \uAE30"
+    "\uC220\uC740 \uD544\uC694\uCE58 \uC54A\uB2E4"
+  };
 
   enum
   {
@@ -37,26 +80,28 @@
 
   static struct  status_
   {
-    int          width;
-    int          height;
+    int  width;
+    int  height;
 
-    int          render_mode;
-    FT_Encoding  encoding;
-    int          res;
-    int          ptsize;            /* current point size */
-    double       gamma;
-    int          angle;
+    int            render_mode;
+    unsigned long  encoding;
+    int            res;
+    int            ptsize;            /* current point size */
+    int            angle;
+    const char*    text;
 
     FTDemo_String_Context  sc;
 
-    FT_Byte      gamma_ramp[256];   /* for show only */
-    FT_Matrix    trans_matrix;
-    int          font_index;
-    char*        header;
-    char         header_buffer[256];
+    FT_Byte    gamma_ramp[256];   /* for show only */
+    FT_Matrix  trans_matrix;
+    int        font_index;
+    char*      header;
+    char       header_buffer[256];
 
   } status = { DIM_X, DIM_Y,
-               RENDER_MODE_STRING, FT_ENCODING_UNICODE, 72, 48, GAMMA, 0 };
+               RENDER_MODE_STRING, FT_ENCODING_UNICODE, 72, 48, 0, NULL,
+               { 0, 0, 0, 0, NULL },
+               { 0 }, { 0, 0, 0, 0 }, 0, NULL, { 0 } };
 
   static FTDemo_Display*  display;
   static FTDemo_Handle*   handle;
@@ -108,16 +153,16 @@
     grLn();
     grWriteln( "  F1 or ?   : display this help screen" );
     grLn();
-    grWriteln( "  a         : toggle anti-aliasing" );
     grWriteln( "  b         : toggle embedded bitmaps (and disable rotation)" );
     grWriteln( "  f         : toggle forced auto-hinting" );
     grWriteln( "  h         : toggle outline hinting" );
     grLn();
     grWriteln( "  1-2       : select rendering mode" );
-    grWriteln( "  l         : cycle through LCD modes" );
+    grWriteln( "  l         : cycle through anti-aliasing modes" );
     grWriteln( "  k         : cycle through kerning modes" );
     grWriteln( "  t         : cycle through kerning degrees" );
-    grWriteln( "  space     : cycle through color" );
+    grWriteln( "  Space     : cycle through color" );
+    grWriteln( "  Tab       : cycle through sample strings" );
     grWriteln( "  V         : toggle vertical rendering" );
     grLn();
     grWriteln( "  g         : increase gamma by 0.1" );
@@ -156,7 +201,7 @@
     FTDemo_Set_Current_Charsize( handle, status.ptsize, status.res );
     FTDemo_Update_Current_Flags( handle );
 
-    FTDemo_String_Set( handle, Text );
+    FTDemo_String_Set( handle, status.text );
   }
 
 
@@ -205,9 +250,15 @@
 
     switch ( handle->lcd_mode )
     {
+    case LCD_MODE_AA:
+      lcd_mode = " normal AA";
+      break;
     case LCD_MODE_LIGHT:
       lcd_mode = " light AA";
       break;
+    case LCD_MODE_LIGHT_SUBPIXEL:
+      lcd_mode = " light AA (subpixel positioning)";
+      break;
     case LCD_MODE_RGB:
       lcd_mode = " LCD (horiz. RGB)";
       break;
@@ -221,8 +272,8 @@
       lcd_mode = " LCD (vert. BGR)";
       break;
     default:
-      handle->lcd_mode = 0;
-      lcd_mode = " normal AA";
+      handle->lcd_mode = LCD_MODE_MONO;
+      lcd_mode = " monochrome";
     }
 
     sprintf( status.header_buffer, "mode changed to %s", lcd_mode );
@@ -246,26 +297,35 @@
   }
 
 
+  static void
+  event_text_change( void )
+  {
+    static int  i = 0;
+
+    status.text = Sample[i];
+
+    i++;
+    if ( i >= (int)( sizeof( Sample ) / sizeof( Sample[0] ) ) )
+      i = 0;
+  }
+
   static void
   event_gamma_change( double  delta )
   {
     int     i;
-    double  gamma_inv;
 
 
-    status.gamma += delta;
+    display->gamma += delta;
 
-    if ( status.gamma > 3.0 )
-      status.gamma = 3.0;
-    else if ( status.gamma < 0.1 )
-      status.gamma = 0.1;
+    if ( display->gamma > 3.0 )
+      display->gamma = 3.0;
+    else if ( display->gamma < 0.1 )
+      display->gamma = 0.1;
 
-    grSetGlyphGamma( status.gamma );
-
-    gamma_inv = 1. / status.gamma;
+    grSetGlyphGamma( display->gamma );
 
     for ( i = 0; i < 256; i++ )
-      status.gamma_ramp[i] = (FT_Byte)( pow( (double)i / 255., gamma_inv )
+      status.gamma_ramp[i] = (FT_Byte)( pow( (double)i / 255., display->gamma )
                                         * 255. + 0.5 );
   }
 
@@ -335,15 +395,6 @@
       event_help();
       break;
 
-    case grKEY( 'a' ):
-      handle->antialias = !handle->antialias;
-      status.header     = handle->antialias
-                          ? (char *)"anti-aliasing is now on"
-                          : (char *)"anti-aliasing is now off";
-
-      FTDemo_Update_Current_Flags( handle );
-      break;
-
     case grKEY( 'b' ):
       handle->use_sbits = !handle->use_sbits;
       status.header     = handle->use_sbits
@@ -403,6 +454,11 @@
       event_color_change();
       break;
 
+    case grKeyTab:
+      event_text_change();
+      FTDemo_String_Set( handle, status.text );
+      break;
+
     case grKEY( 'V' ):
       sc->vertical  = !sc->vertical;
       status.header = sc->vertical
@@ -460,60 +516,19 @@
     y = ( bitmap->rows + 256 ) / 2;
 
     for (i = 0; i < 256; i++)
-      p[bitmap->pitch * ( y - gamma_ramp[i] ) + bpp * ( x + i )] ^= 0xFF;
+      p[bitmap->pitch * ( y - i ) + bpp * ( x + gamma_ramp[i] )] ^= 0xFF;
   }
 
 
   static void
   write_header( FT_Error  error_code )
   {
-    FT_Face      face;
-    const char*  basename;
+    FTDemo_Draw_Header( handle, display, status.ptsize, status.res,
+                        -1, error_code );
 
-
-    error = FTC_Manager_LookupFace( handle->cache_manager,
-                                    handle->scaler.face_id, &face );
-    if ( error )
-      PanicZ( "can't access font file" );
-
-    if ( !status.header )
-    {
-      basename = ft_basename( handle->current_font->filepathname );
-
-      switch ( error_code )
-      {
-      case FT_Err_Ok:
-        sprintf( status.header_buffer,
-                 "%.50s %.50s (file `%.100s')", face->family_name,
-                 face->style_name, basename );
-        break;
-
-      case FT_Err_Invalid_Pixel_Size:
-        sprintf( status.header_buffer, "Invalid pixel size (file `%.100s')",
-                 basename );
-        break;
-
-      case FT_Err_Invalid_PPem:
-        sprintf( status.header_buffer, "Invalid ppem value (file `%.100s')",
-                 basename );
-        break;
-
-      default:
-        sprintf( status.header_buffer, "File `%.100s': error 0x%04x", basename,
-            (FT_UShort)error_code );
-        break;
-      }
-
-      status.header = status.header_buffer;
-    }
-
-    grWriteCellString( display->bitmap, 0, 0,
-                       status.header, display->fore_color );
-
-    sprintf( status.header_buffer, "at %g points, angle = %d, gamma = %g",
-             status.ptsize / 64.0, status.angle, status.gamma );
-    grWriteCellString( display->bitmap, 0, CELLSTRING_HEIGHT,
-                       status.header_buffer, display->fore_color );
+    if ( status.header )
+      grWriteCellString( display->bitmap, 0, 2 * HEADER_HEIGHT,
+                         status.header, display->fore_color );
 
     grRefreshSurface( display->surface );
   }
@@ -592,7 +607,7 @@
       case 'm':
         if ( *argc < 3 )
           usage( execname );
-        Text = optarg;
+        status.text = optarg;
         break;
 
       case 'r':
@@ -663,7 +678,7 @@
 
     for ( ; argc > 0; argc--, argv++ )
     {
-      error = FTDemo_Install_Font( handle, argv[0], 0 );
+      error = FTDemo_Install_Font( handle, argv[0], 0, 0 );
 
       if ( error )
       {
@@ -687,10 +702,14 @@
     grSetTitle( display->surface,
                 "FreeType String Viewer - press ? for help" );
 
+    status.header = NULL;
+
+    if ( !status.text )
+      event_text_change();
+
     event_color_change();
     event_gamma_change( 0 );
     event_font_change( 0 );
-    status.header = 0;
 
     do
     {
