@@ -32,6 +32,7 @@
 #include FT_INTERNAL_OBJECTS_H
 
 #include "common.h"
+#include "mlgetopt.h"
 
 
 #include <stdio.h>
@@ -47,7 +48,7 @@
     GX_VALIDATE,
     CKERN_VALIDATE,
 
-    LAST_VALIDATE,
+    LAST_VALIDATE
 
   } ValidatorType;
   static ValidatorType  validator;
@@ -98,8 +99,8 @@
     int         (* is_implemented)( FT_Library library );
 
     FT_Error    (* run)           ( FT_Face     face,
-				    const char* tables,
-				    int         validation_level );
+                                    const char* tables,
+                                    int         validation_level );
     int         (* list_tables)   ( FT_Face     face );
 
     TableSpec      table_spec;
@@ -112,14 +113,14 @@
   static int is_ckern_validator_implemented ( FT_Library library );
 
   static FT_Error run_ot_validator          ( FT_Face      face,
-					      const char*  tables,
-					      int          validation_level );
+                                              const char*  tables,
+                                              int          validation_level );
   static FT_Error run_gx_validator          ( FT_Face      face,
-					      const char*  tables,
-					      int          validation_level );
+                                              const char*  tables,
+                                              int          validation_level );
   static FT_Error run_ckern_validator       ( FT_Face      face,
-					      const char*  dialect_request,
-					      int          validation_level );
+                                              const char*  dialect_request,
+                                              int          validation_level );
 
   static int list_ot_tables                 ( FT_Face  face );
   static int list_gx_tables                 ( FT_Face  face );
@@ -132,9 +133,9 @@
       OT_VALIDATE,
       "ot",
       ( "FT_OpenType_Validate"
-	" is disabled!  Recompile FreeType 2 with "
-	"otvalid"
-	" module enabled.\n" ),
+        " is disabled!  Recompile FreeType 2 with "
+        "otvalid"
+        " module enabled.\n" ),
       is_ot_validator_implemented,
       run_ot_validator,
       list_ot_tables,
@@ -145,9 +146,9 @@
       GX_VALIDATE,
       "gx",
       ( "FT_TrueTypeGX_Validate"
-	" is disabled!  Recompile FreeType 2 with "
-	"gxvalid"
-	" module enabled.\n" ),
+        " is disabled!  Recompile FreeType 2 with "
+        "gxvalid"
+        " module enabled.\n" ),
       is_gx_validator_implemented,
       run_gx_validator,
       list_gx_tables,
@@ -158,9 +159,9 @@
       CKERN_VALIDATE,
       "ckern",
       ( "FT_ClassicKern_Validate"
-	" is disabled!  Recompile FreeType 2 with "
-	"gxvalid"		/* NOTE: classic kern validator is in gxvalid. */
-	" module enabled.\n" ),
+        " is disabled!  Recompile FreeType 2 with "
+        "gxvalid"                /* NOTE: classic kern validator is in gxvalid. */
+        " module enabled.\n" ),
       is_ckern_validator_implemented,
       run_ckern_validator,
       list_ckern_tables,
@@ -204,7 +205,7 @@
 
 
   /* To initialize the internal variable, call this
-     funtion with FT_Library variable. Then call
+     function with FT_Library variable. Then call
      with NULL. The print messages is printed if
      call with NULL. */
   static void
@@ -255,7 +256,7 @@
       "\n" );
 
     fprintf( stderr,
-      "  -T tbls       [ot, gx] Select snft table name tags to be validated.\n"
+      "  -T tbls       [ot, gx] Select sfnt table name tags to be validated.\n"
       "                Use `:' to separate tags.\n"
       "\n" );
     for ( i = 0; i < LAST_VALIDATE; i++ )
@@ -263,7 +264,7 @@
       v = &validators[i];
 
       if ( v->n_table_spec == 0 )
-	continue;
+        continue;
 
       fprintf( stderr,
         "                Supported tables in %s validator:\n"
@@ -271,8 +272,8 @@
                v->symbol );
       for ( j = 0; j < v->n_table_spec; j++ )
       {
-	fprintf( stderr, " " );
-	print_tag( stderr, v->table_spec[j].tag );
+        fprintf( stderr, " " );
+        print_tag( stderr, v->table_spec[j].tag );
       }
       fprintf( stderr,
         "\n"
@@ -515,15 +516,15 @@
     {
       if ( spec[i].validation_flag & validation_flags )
       {
-	n_targets++;
+        n_targets++;
 
-	if ( data[i] != NULL )
-	{
-	  printf( "[%s:%s] ", execname, validators[validator].symbol );
-	  print_tag( stdout, spec[i].tag );
-	  printf( "%s", "...pass\n" );
-	  n_passes++;
-	}
+        if ( data[i] != NULL )
+        {
+          printf( "[%s:%s] ", execname, validators[validator].symbol );
+          print_tag( stdout, spec[i].tag );
+          printf( "%s", "...pass\n" );
+          n_passes++;
+        }
       }
     }
 
@@ -540,7 +541,7 @@
 
 
   /*
-   * OpenType related funtions
+   * OpenType related functions
    */
   static int
   is_ot_validator_implemented( FT_Library library )
@@ -599,7 +600,7 @@
 
 
   /*
-   * TrueTypeGX related funtions
+   * TrueTypeGX related functions
    */
   static int
   is_gx_validator_implemented( FT_Library library )
@@ -658,7 +659,7 @@
 
 
   /*
-   * Classic kern related funtions
+   * Classic kern related functions
    */
   static int
   is_ckern_validator_implemented( FT_Library library )
@@ -786,7 +787,7 @@
         {
           int i;
 
-	  validator = LAST_VALIDATE;
+          validator = LAST_VALIDATE;
           for ( i = 0; i < LAST_VALIDATE; i++ )
           {
             if ( strcmp( optarg, validators[i].symbol ) == 0 )
@@ -795,11 +796,11 @@
               break;
             }
           }
-	  if ( validator == LAST_VALIDATE )
-	  {
-	    fprintf( stderr, "*** Unknown validator name: %s\n", optarg );
-	    print_usage( NULL );
-	  }
+          if ( validator == LAST_VALIDATE )
+          {
+            fprintf( stderr, "*** Unknown validator name: %s\n", optarg );
+            print_usage( NULL );
+          }
         }
         break;
 
@@ -873,8 +874,8 @@
       status = 0;
 
       if ( !validators[validator].is_implemented( library ) )
-	panic( FT_Err_Unimplemented_Feature,
-	       validators[validator].unimplemented_message );
+        panic( FT_Err_Unimplemented_Feature,
+               validators[validator].unimplemented_message );
 
 
       /* TODO: Multiple faces in a font file? */
@@ -883,9 +884,9 @@
         panic( error, "Could not open face." );
 
       if ( dump_table_list )
-	validators[validator].list_tables( face );
+        validators[validator].list_tables( face );
       else
-	status = validators[validator].run( face, tables, validation_level );
+        status = validators[validator].run( face, tables, validation_level );
 
       FT_Done_Face( face );
       FT_Done_FreeType( library );
